@@ -3,6 +3,7 @@ import time as ttime
 import json
 import logging
 import multiprocessing
+import pprint
 import threading
 import asyncio
 import zmq
@@ -28,7 +29,7 @@ def count_threads_with_name(name):
     """
     n_count = 0
     for th in threading.enumerate():
-        if th.name == name:
+        if th.name.startswith(name):
             n_count += 1
     return n_count
 
@@ -95,7 +96,7 @@ def test_PipeJsonRpcReceive_1():
 
     pc = PipeJsonRpcReceive(conn=conn2, name=new_name)
     pc.start()
-    assert count_threads_with_name(new_name) == 1, "One thread is expected to exist"
+    assert count_threads_with_name(new_name) == 2, "Two threads are expected to exist"
 
     pc.start()  # Expected to do nothing
 
@@ -104,7 +105,7 @@ def test_PipeJsonRpcReceive_1():
     assert count_threads_with_name(new_name) == 0, "No threads are expected to exist"
 
     pc.start()  # Restart
-    assert count_threads_with_name(new_name) == 1, "One thread is expected to exist"
+    assert count_threads_with_name(new_name) == 2, "Two thread are expected to exist"
     pc.stop()
 
 
@@ -230,7 +231,8 @@ def test_PipeJsonRpcReceive_3():
 
 def test_PipeJsonRpcReceive_4():
     """
-    Test if all outdated unprocessed messages are deleted once the processing thread is started.
+    Test if all outdated unprocessed messages are deleted from the pipe
+    as the processing thread is started.
     """
 
     def method_handler3(*, value=3):
@@ -272,7 +274,52 @@ def test_PipeJsonRpcReceive_4():
     pc.stop()
 
 
-def test_PipeJsonRpcReceive_5_failing():
+# fmt: off
+@pytest.mark.parametrize("clear_buffer", [False, True])
+# fmt: on
+def test_PipeJsonRpcReceive_5(clear_buffer):
+    """
+    Checking that the buffer overflow does not overflow the pipe.
+    """
+    n_calls = 0
+    start_processing = False
+
+    def method_handler3():
+        nonlocal n_calls
+        while not start_processing:
+            ttime.sleep(0.05)
+        n_calls += 1
+
+    conn1, conn2 = multiprocessing.Pipe()
+    pc = PipeJsonRpcReceive(conn=conn2)
+    pc.add_method(method_handler3, "method3")
+    pc.start()
+
+    # Non-existing method ('Method not found' error)
+    request = format_jsonrpc_msg("method3")
+
+    n_buf = pc._msg_recv_buffer_size
+
+    conn1.send(json.dumps(request))
+    ttime.sleep(1)
+    for _ in range(n_buf * 2):
+        conn1.send(json.dumps(request))
+
+    ttime.sleep(1)
+
+    if clear_buffer:
+        ttime.sleep(1)
+        pc.clear_buffer()
+
+    start_processing = True
+
+    ttime.sleep(1)
+    assert n_calls == (1 if clear_buffer else (n_buf + 1))
+
+    pc.stop()
+
+
+def test_PipeJsonRpcReceive_6_failing():
     """
     Those tests are a result of exploration of how `json-rpc` error processing works.
     """
@@ -337,7 +384,7 @@ def test_PipeJsonRpcReceive_5_failing():
     pc.stop()
 
 
-def test_PipeJsonRpcReceive_6_failing():
+def test_PipeJsonRpcReceive_7_failing():
     """
     Exception is raised inside the handler is causing 'Server Error' -32000.
     Returns error type (Exception type) and message. It is questionable whether
@@ -371,7 +418,7 @@ def test_PipeJsonRpcReceive_6_failing():
     pc.stop()
 
 
-def test_PipeJsonRpcReceive_7_failing():
+def test_PipeJsonRpcReceive_8_failing():
     """
     This is an example of handler to timeout. 'json-rpc' package can not handle timeouts.
     Care must be taken to write handles that execute quickly. Timeouts must be handled
@@ -417,7 +464,7 @@ def test_PipeJsonRpcSendAsync_1():
 
         pc = PipeJsonRpcSendAsync(conn=conn1, name=new_name)
         pc.start()
-        assert count_threads_with_name(new_name) == 1, "One thread is expected to exist"
+        assert count_threads_with_name(new_name) == 2, "One thread is expected to exist"
 
         pc.start()  # Expected to do nothing
 
@@ -426,7 +473,7 @@ def test_PipeJsonRpcSendAsync_1():
         assert count_threads_with_name(new_name) == 0, "No threads are expected to exist"
 
         pc.start()  # Restart
-        assert count_threads_with_name(new_name) == 1, "One thread is expected to exist"
+        assert count_threads_with_name(new_name) == 2, "One thread is expected to exist"
         pc.stop()
 
     asyncio.run(object_start_stop())
@@ -622,7 +669,7 @@ class _PipeJsonRpcReceiveTest(PipeJsonRpcReceive):
     The test emulates possible issues with the process receiving messages.
     """
 
-    def _conn_received(self, msg):
+    def _handle_msg(self, msg):
         response = JSONRPCResponseManager.handle(msg, self._dispatcher)
         if response:
             response = response.json
@@ -662,7 +709,7 @@ def test_PipeJsonRpcSendAsync_6(caplog):
     asyncio.run(send_messages())
     pc.stop()
 
-    txt = "Unsolicited message received"
+    txt = "Unexpected message received"
     assert txt in caplog.text
     assert caplog.text.count(txt) == 2, caplog.text
 
@@ -1087,7 +1134,48 @@ def test_ZMQCommSendThreads_4(is_blocking, raise_exception, delay_between_reads,
     thread.join()
 
 
-def test_ZMQCommSendThreads_5_fail():
+def _zmq_server_delay3(delay):
+    """
+    ZMQ server: delay of 3 seconds before the response
+    """
+    ctx = zmq.Context()
+    zmq_socket = ctx.socket(zmq.REP)
+    zmq_socket.bind("tcp://*:60615")
+    msg_in = zmq_socket.recv_json()
+    ttime.sleep(delay)  # Generate timeout at the client
+    msg_out = {"success": True, "msg_in": msg_in}
+    zmq_socket.send_json(msg_out)
+    zmq_socket.close(linger=10)
+
+
+# fmt: off
+@pytest.mark.parametrize("timeout", [None, 2000, 5000])
+# fmt: on
+def test_ZMQCommSendThreads_5(timeout):
+    """
+    ZMQCommSendThreads: Pass timeout as 'send_message' parameter.
+    """
+    delay = 3000
+    thread = threading.Thread(target=_zmq_server_delay3, kwargs={"delay": delay / 1000})
+    thread.start()
+
+    timeout_default = 2000
+    zmq_comm = ZMQCommSendThreads(timeout_recv=timeout_default)
+    method, params = "testing", {"p1": 10, "p2": "abc"}
+
+    timeout_used = timeout or timeout_default
+
+    if timeout_used < delay:
+        with pytest.raises(CommTimeoutError, match="timeout occurred"):
+            zmq_comm.send_message(method=method, params=params, timeout=timeout, raise_exceptions=True)
+    else:
+        msg_recv = zmq_comm.send_message(method=method, params=params, timeout=timeout, raise_exceptions=True)
+        assert msg_recv["success"] is True, pprint.pformat(msg_recv)
+
+    thread.join()
+
+
+def test_ZMQCommSendThreads_6_fail():
     """
     Invalid public key
     """
@@ -1256,7 +1344,39 @@ def test_ZMQCommSendAsync_4(raise_exception, delay_between_reads, encryption_ena
     thread.join()
 
 
-def test_ZMQCommSendAsync_5_fail():
+# fmt: off
+@pytest.mark.parametrize("timeout", [None, 2000, 5000])
+# fmt: on
+def test_ZMQCommSendAsync_5(timeout):
+    """
+    ZMQCommSendThreads: Pass timeout as 'send_message' parameter.
+    """
+    delay = 3000
+    thread = threading.Thread(target=_zmq_server_delay3, kwargs={"delay": delay / 1000})
+    thread.start()
+
+    async def testing():
+        timeout_default = 2000
+        zmq_comm = ZMQCommSendAsync(timeout_recv=timeout_default)
+        method, params = "testing", {"p1": 10, "p2": "abc"}
+
+        timeout_used = timeout or timeout_default
+
+        if timeout_used < delay:
+            with pytest.raises(CommTimeoutError, match="timeout occurred"):
+                await zmq_comm.send_message(method=method, params=params, timeout=timeout, raise_exceptions=True)
+        else:
+            msg_recv = await zmq_comm.send_message(
+                method=method, params=params, timeout=timeout, raise_exceptions=True
+            )
+            assert msg_recv["success"] is True, pprint.pformat(msg_recv)
+
+    asyncio.run(testing())
+
+    thread.join()
+
+
+def test_ZMQCommSendAsync_6_fail():
     """
     Invalid public key
     """
